@@ -2,8 +2,9 @@ import asyncio
 import functools
 import inspect
 import logging
+from collections.abc import Callable
 from functools import partial
-from typing import TYPE_CHECKING, Any, Callable, Optional, Union
+from typing import TYPE_CHECKING, Any, Optional
 
 from slixmpp import JID, Iq
 from slixmpp.exceptions import XMPPError
@@ -154,7 +155,7 @@ class AdhocProvider:
             handler()
 
     @staticmethod
-    async def __wrap_handler(f: Union[Callable, functools.partial], *a, **k):  # type: ignore
+    async def __wrap_handler(f: Callable | functools.partial, *a, **k):  # type: ignore
         try:
             if inspect.iscoroutinefunction(f):
                 return await f(*a, **k)
@@ -213,7 +214,7 @@ class AdhocProvider:
 
         return await self.__handle_result(session, result, adhoc_session)
 
-    def register(self, command: Command, jid: Optional[JID] = None) -> None:
+    def register(self, command: Command, jid: JID | None = None) -> None:
         """
         Register a command as a adhoc command.
 
