@@ -3,7 +3,7 @@ import functools
 import importlib
 import logging
 from datetime import datetime
-from typing import Any, Optional
+from typing import Any
 
 from slixmpp import JID
 from slixmpp.exceptions import XMPPError
@@ -124,7 +124,10 @@ class DeleteUser(AdminCommand):
         )
 
     async def delete(
-        self, form_values: FormValues, _session: AnyBaseSession, _ifrom: JID
+        self,
+        form_values: FormValues,
+        _session: AnyBaseSession | None,
+        _ifrom: JID,
     ) -> Confirmation:
         jid: JID = form_values.get("jid")  # type:ignore
         with self.xmpp.store.session() as orm:
@@ -139,7 +142,7 @@ class DeleteUser(AdminCommand):
         )
 
     async def finish(
-        self, _session: Optional[AnyBaseSession], _ifrom: JID, jid: JID
+        self, _session: AnyBaseSession | None, _ifrom: JID, jid: JID
     ) -> None:
         with self.xmpp.store.session() as orm:
             user = orm.query(GatewayUser).where(GatewayUser.jid == jid).one_or_none()
@@ -176,7 +179,9 @@ class ChangeLoglevel(AdminCommand):
 
     @staticmethod
     async def finish(
-        form_values: FormValues, _session: AnyBaseSession, _ifrom: JID
+        form_values: FormValues,
+        _session: AnyBaseSession | None,
+        _ifrom: JID,
     ) -> None:
         logging.getLogger().setLevel(int(form_values["level"]))  # type:ignore
 
