@@ -1,15 +1,11 @@
 from abc import ABC
+from collections.abc import Awaitable, Callable, Iterable, Sequence
 from dataclasses import dataclass, field
 from enum import Enum
 from typing import (
     TYPE_CHECKING,
     Any,
-    Awaitable,
-    Callable,
-    Iterable,
-    Optional,
-    Sequence,
-    Type,
+    Generic,
     TypedDict,
     Union,
 )
@@ -21,32 +17,21 @@ from slixmpp.plugins.xep_0004 import FormField as SlixFormField
 from slixmpp.types import JidStr
 
 from ..core import config
-from ..util.types import AnyBaseSession, FieldType
+from ..util.types import AnyBaseSession, FieldType, SessionType
 
 NODE_PREFIX = "https://slidge.im/command/core/"
 
 if TYPE_CHECKING:
     from ..core.gateway import BaseGateway
-    from ..core.session import BaseSession
     from .categories import CommandCategory
 
 
-HandlerType = Union[
-    Callable[[AnyBaseSession, JID], "CommandResponseType"],
-    Callable[[AnyBaseSession, JID], Awaitable["CommandResponseType"]],
-]
+HandlerType = (
+    Callable[[AnyBaseSession, JID], "CommandResponseType"]
+    | Callable[[AnyBaseSession, JID], Awaitable["CommandResponseType"]]
+)
 
-FormValues = dict[str, Union[str, JID, bool]]
-
-
-FormHandlerType = Callable[
-    [FormValues, AnyBaseSession, JID],
-    Awaitable["CommandResponseType"],
-]
-
-ConfirmationHandlerType = Callable[
-    [Optional[AnyBaseSession], JID], Awaitable["CommandResponseType"]
-]
+FormValues = dict[str, str | JID | bool]
 
 
 @dataclass
@@ -59,7 +44,7 @@ class TableResult:
     """
     The 'columns names' of the table.
     """
-    items: Sequence[dict[str, Union[str, JID]]]
+    items: Sequence[dict[str, str | JID]]
     """
     The rows of the table. Each row is a dict where keys are the fields ``var``
     attribute.
@@ -99,7 +84,7 @@ class SearchResult(TableResult):
 
 
 @dataclass
-class Confirmation:
+class Confirmation(Generic[SessionType]):
     """
     A confirmation 'dialog'
     """
@@ -108,11 +93,14 @@ class Confirmation:
     """
     The text presented to the command triggering user
     """
-    handler: ConfirmationHandlerType
+    handler: Callable[
+        [SessionType | None, JID],
+        Awaitable["CommandResponseType"],
+    ]
     """
     An async function that should return a ResponseType
     """
-    success: Optional[str] = None
+    success: str | None = None
     """
     Text in case of success, used if handler does not return anything
     """
@@ -143,7 +131,7 @@ class Confirmation:
 
 
 @dataclass
-class Form:
+class Form(Generic[SessionType]):
     """
     A form, to request user input
     """
@@ -151,14 +139,17 @@ class Form:
     title: str
     instructions: str
     fields: Sequence["FormField"]
-    handler: FormHandlerType
+    handler: Callable[
+        [FormValues, SessionType | None, JID],
+        Awaitable["CommandResponseType"],
+    ]
     handler_args: Iterable[Any] = field(default_factory=list)
     handler_kwargs: dict[str, Any] = field(default_factory=dict)
     timeout_handler: Callable[[], None] | None = None
 
     def get_values(
         self, slix_form: SlixForm
-    ) -> dict[str, Union[list[str], list[JID], str, JID, bool, None]]:
+    ) -> dict[str, list[str] | list[JID] | str | JID | bool | None]:
         """
         Parse form submission
 
@@ -221,7 +212,7 @@ class FormField:
     """
     Internal name of the field, will be used to retrieve via :py:attr:`slidge.GatewayUser.registration_form`
     """
-    label: Optional[str] = None
+    label: str | None = None
     """Description of the field that the user will see"""
     required: bool = False
     """Whether this field is mandatory or not"""
@@ -234,9 +225,9 @@ class FormField:
     """Type of the field, see `XEP-0004 <https://xmpp.org/extensions/xep-0004.html#protocol-fieldtypes>`_"""
     value: str = ""
     """Pre-filled value. Will be automatically pre-filled if a registered user modifies their subscription"""
-    options: Optional[list[Option]] = None
+    options: list[Option] | None = None
 
-    image_url: Optional[str] = None
+    image_url: str | None = None
     """An image associated to this field, eg, a QR code"""
 
     def __post_init__(self) -> None:
@@ -249,8 +240,8 @@ class FormField:
         return [x["value"] for x in self.options]
 
     def validate(
-        self, value: Optional[Union[str, list[str]]]
-    ) -> Union[list[str], list[JID], str, JID, bool, None]:
+        self, value: str | list[str] | None
+    ) -> list[str] | list[JID] | str | JID | bool | None:
         """
         Raise appropriate XMPPError if a given value is valid for this field
 
@@ -260,10 +251,12 @@ class FormField:
         if isinstance(value, list) and not self.type.endswith("multi"):
             raise XMPPError("not-acceptable", "A single value was expected")
 
-        if self.type in ("list-multi", "jid-multi"):
+        if self.type in ("list-multi", "jid-multi", "text-multi"):
             if not value:
                 value = []
             if isinstance(value, list):
+                if self.type == "text-multi":
+                    return value
                 return self.__validate_list_multi(value)
             else:
                 raise XMPPError("not-acceptable", "Multiple values was expected")
@@ -291,17 +284,13 @@ class FormField:
 
         return value
 
-    def __validate_list_multi(self, value: list[str]) -> Union[list[str], list[JID]]:
-        # COMPAT: all the "if v" and "if not v" are workarounds for https://codeberg.org/slidge/slidge/issues/43
-        # They should be reverted once the bug is fixed upstream, cf https://soprani.ca/todo/390
+    def __validate_list_multi(self, value: list[str]) -> list[str] | list[JID]:
         for v in value:
             if v not in self.__acceptable_options():
-                if not v:
-                    continue
                 raise XMPPError("not-acceptable", f"Not a valid option: '{v}'")
         if self.type == "list-multi":
-            return [v for v in value if v]
-        return [JID(v) for v in value if v]
+            return value
+        return [JID(v) for v in value]
 
     def get_xml(self) -> SlixFormField:
         """
@@ -323,10 +312,10 @@ class FormField:
         return f
 
 
-CommandResponseType = Union[TableResult, Confirmation, Form, str, None]
+CommandResponseType = TableResult | Confirmation | Form | str | None
 
 
-class Command(ABC):
+class Command(ABC, Generic[SessionType]):
     """
     Abstract base class to implement gateway commands (chatbot and ad-hoc)
     """
@@ -353,7 +342,7 @@ class Command(ABC):
     Who can use this command
     """
 
-    CATEGORY: Optional[Union[str, "CommandCategory"]] = None
+    CATEGORY: Union[str, "CommandCategory"] | None = None
     """
     If used, the command will be under this top-level category.
     Use the same string for several commands to group them.
@@ -361,7 +350,7 @@ class Command(ABC):
     interface.
     """
 
-    subclasses = list[Type["Command"]]()
+    subclasses = list[type["Command"]]()
 
     def __init__(self, xmpp: "BaseGateway") -> None:
         self.xmpp = xmpp
@@ -372,7 +361,10 @@ class Command(ABC):
         cls.subclasses.append(cls)
 
     async def run(
-        self, session: Optional["BaseSession[Any, Any]"], ifrom: JID, *args: str
+        self,
+        session: SessionType | None,
+        ifrom: JID,
+        *args: str,
     ) -> CommandResponseType:
         """
         Entry point of the command
@@ -386,7 +378,7 @@ class Command(ABC):
         """
         raise XMPPError("feature-not-implemented")
 
-    def _get_session(self, jid: JID) -> Optional["BaseSession[Any, Any]"]:
+    def _get_session(self, jid: JID) -> SessionType | None:
         return self.xmpp.get_session_from_jid(jid)
 
     def __can_use_command(self, jid: JID):
@@ -397,8 +389,8 @@ class Command(ABC):
         self,
         jid: JID,
         fetch_session: bool = True,
-        session: Optional["BaseSession[Any, Any]"] = None,
-    ) -> Optional["BaseSession[Any, Any]"]:
+        session: SessionType | None = None,
+    ) -> SessionType | None:
         """
         Raise an appropriate error is jid is not authorized to use the command
 
