@@ -6,7 +6,8 @@ import asyncio
 import functools
 import inspect
 import logging
-from typing import TYPE_CHECKING, Any, Callable, Literal, Optional, Union, overload
+from collections.abc import Callable
+from typing import TYPE_CHECKING, Any, Literal, overload
 from urllib.parse import quote as url_quote
 
 from slixmpp import JID, CoroutineCallback, Message, StanzaPath
@@ -51,11 +52,11 @@ class ChatCommandProvider:
         self._commands[t] = command
 
     @overload
-    async def input(self, jid: JidStr, text: Optional[str] = None) -> str: ...
+    async def input(self, jid: JidStr, text: str | None = None) -> str: ...
 
     @overload
     async def input(
-        self, jid: JidStr, text: Optional[str] = None, *, blocking: Literal[False] = ...
+        self, jid: JidStr, text: str | None = None, *, blocking: Literal[False] = ...
     ) -> asyncio.Future[str]: ...
 
     @overload
@@ -113,7 +114,7 @@ class ChatCommandProvider:
             return f
         try:
             await asyncio.wait_for(f, timeout)
-        except asyncio.TimeoutError:
+        except TimeoutError:
             self.xmpp.send_message(
                 mto=jid,
                 mbody="You took too much time to reply",
@@ -270,7 +271,7 @@ class ChatCommandProvider:
         return await self._handle_result(result, msg, session)
 
     @staticmethod
-    async def __wrap_handler(msg, f: Union[Callable, functools.partial], *a, **k):
+    async def __wrap_handler(msg, f: Callable | functools.partial, *a, **k):
         try:
             if inspect.iscoroutinefunction(f):
                 return await f(*a, **k)
