@@ -173,7 +173,7 @@ class Register(Command):
                 self.xmpp.qr_pending_registrations[user.jid.bare],  # type:ignore
                 config.QR_TIMEOUT,
             )
-        except asyncio.TimeoutError:
+        except TimeoutError:
             raise XMPPError(
                 "remote-server-timeout",
                 (
