@@ -1,8 +1,9 @@
 import datetime
 import logging
 import warnings
+from collections.abc import Iterable, Iterator
 from datetime import date
-from typing import TYPE_CHECKING, Generic, Iterable, Iterator, Optional, Union
+from typing import TYPE_CHECKING, Generic
 from xml.etree import ElementTree as ET
 
 from slixmpp import JID, Message, Presence
@@ -29,7 +30,7 @@ class LegacyContact(
     FullCarbonMixin,
     ReactionRecipientMixin,
     ThreadRecipientMixin,
-    metaclass=SubclassableOnce,
+    SubclassableOnce,
 ):
     """
     This class centralizes actions in relation to a specific legacy contact.
@@ -201,7 +202,7 @@ class LegacyContact(
         else:
             return
 
-        last_seen: Optional[datetime.datetime] = (
+        last_seen: datetime.datetime | None = (
             stanza["idle"]["since"] if stanza.get_plugin("idle", check=True) else None
         )
 
@@ -291,7 +292,7 @@ class LegacyContact(
         return self.stored.nick or ""
 
     @name.setter
-    def name(self, n: Optional[str]) -> None:
+    def name(self, n: str | None) -> None:
         if self.stored.nick == n:
             return
         self.update_stored_attribute(nick=n)
@@ -317,18 +318,18 @@ class LegacyContact(
     def set_vcard(
         self,
         /,
-        full_name: Optional[str] = None,
-        given: Optional[str] = None,
-        surname: Optional[str] = None,
-        birthday: Optional[date] = None,
-        phone: Optional[str] = None,
+        full_name: str | None = None,
+        given: str | None = None,
+        surname: str | None = None,
+        birthday: date | None = None,
+        phone: str | None = None,
         phones: Iterable[str] = (),
-        note: Optional[str] = None,
-        url: Optional[str] = None,
-        email: Optional[str] = None,
-        country: Optional[str] = None,
-        locality: Optional[str] = None,
-        pronouns: Optional[str] = None,
+        note: str | None = None,
+        url: str | None = None,
+        email: str | None = None,
+        country: str | None = None,
+        locality: str | None = None,
+        pronouns: str | None = None,
     ) -> None:
         """
         Update xep:`0292` data for this contact.
@@ -437,11 +438,11 @@ class LegacyContact(
                 nick,
             )
 
-    def send_friend_request(self, text: Optional[str] = None) -> None:
+    def send_friend_request(self, text: str | None = None) -> None:
         presence = self._make_presence(ptype="subscribe", pstatus=text, bare=True)
         self._send(presence, nick=True)
 
-    async def accept_friend_request(self, text: Optional[str] = None) -> None:
+    async def accept_friend_request(self, text: str | None = None) -> None:
         """
         Call this to signify that this Contact has accepted to be a friend
         of the user.
@@ -457,7 +458,7 @@ class LegacyContact(
         await self.__broadcast_pubsub_items()
         self.log.debug("Accepted friend request")
 
-    def reject_friend_request(self, text: Optional[str] = None) -> None:
+    def reject_friend_request(self, text: str | None = None) -> None:
         """
         Call this to signify that this Contact has refused to be a contact
         of the user (or that they don't want to be friends anymore)
@@ -545,9 +546,9 @@ class LegacyContact(
     def _make_presence(
         self,
         *,
-        last_seen: Optional[datetime.datetime] = None,
-        status_codes: Optional[set[int]] = None,
-        user_full_jid: Optional[JID] = None,
+        last_seen: datetime.datetime | None = None,
+        status_codes: set[int] | None = None,
+        user_full_jid: JID | None = None,
         **presence_kwargs,
     ):
         p = super()._make_presence(last_seen=last_seen, **presence_kwargs)
@@ -559,7 +560,7 @@ class LegacyContact(
         return p
 
 
-def is_markable(stanza: Union[Message, Presence]):
+def is_markable(stanza: Message | Presence):
     if isinstance(stanza, Presence):
         return False
     return bool(stanza["body"])
