@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import logging
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING
 
 from slixmpp import JID, Iq
 from slixmpp.exceptions import XMPPError
@@ -17,7 +17,7 @@ if TYPE_CHECKING:
 class RegistrationMixin(DispatcherMixin):
     __slots__: list[str] = []
 
-    def __init__(self, xmpp: "BaseGateway") -> None:
+    def __init__(self, xmpp: BaseGateway) -> None:
         super().__init__(xmpp)
         xmpp["xep_0077"].api.register(
             self.xmpp.make_registration_form, "make_registration_form"
@@ -63,7 +63,7 @@ class RegistrationMixin(DispatcherMixin):
             log.info("New user: %s", user)
 
     async def _user_modify(
-        self, _gateway_jid, _node, ifrom: JID, form_dict: dict[str, Optional[str]]
+        self, _gateway_jid, _node, ifrom: JID, form_dict: dict[str, str | None]
     ):
         await self.xmpp.user_prevalidate(ifrom, form_dict)
         log.debug("Modify user: %s", ifrom)
