@@ -2,14 +2,16 @@
 This module extends slixmpp.ComponentXMPP to make writing new LegacyClients easier
 """
 
+import abc
 import asyncio
 import logging
 import re
 import tempfile
+from collections.abc import Callable, Sequence
 from copy import copy
 from datetime import datetime
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Callable, Optional, Sequence, Type, Union, cast
+from typing import TYPE_CHECKING, Any, cast
 
 import aiohttp
 import qrcode
@@ -40,7 +42,7 @@ from slidge.db.avatar import CachedAvatar, avatar_cache
 from slidge.db.meta import JSONSerializable
 from slidge.slixfix.delivery_receipt import DeliveryReceipt
 from slidge.slixfix.roster import RosterBackend
-from slidge.util import ABCSubclassableOnceAtMost
+from slidge.util import SubclassableOnce
 from slidge.util.types import Avatar, MessageOrPresenceTypeVar
 from slidge.util.util import timeit
 
@@ -51,7 +53,8 @@ if TYPE_CHECKING:
 class BaseGateway(
     ComponentXMPP,
     MessageMixin,
-    metaclass=ABCSubclassableOnceAtMost,
+    SubclassableOnce,
+    abc.ABC,
 ):
     """
     The gateway component, handling registrations and un-registrations.
@@ -101,7 +104,7 @@ class BaseGateway(
     """Name of the component, as seen in service discovery by XMPP clients"""
     COMPONENT_TYPE: str = ""
     """Type of the gateway, should follow https://xmpp.org/registrar/disco-categories.html"""
-    COMPONENT_AVATAR: Optional[Avatar | Path | str] = None
+    COMPONENT_AVATAR: Avatar | Path | str | None = None
     """
     Path, bytes or URL used by the component as an avatar.
     """
@@ -344,13 +347,13 @@ class BaseGateway(
                 config.USER_JID_VALIDATOR,
             )
         self.jid_validator: re.Pattern = re.compile(config.USER_JID_VALIDATOR)
-        self.qr_pending_registrations = dict[str, asyncio.Future[Optional[dict]]]()
+        self.qr_pending_registrations = dict[str, asyncio.Future[dict | None]]()
 
         self.register_plugins()
         self.__setup_legacy_module_subclasses()
 
         self.get_session_from_stanza: Callable[
-            [Union[Message, Presence, Iq]], BaseSession
+            [Message | Presence | Iq], BaseSession
         ] = self._session_cls.from_stanza
         self.get_session_from_user: Callable[[GatewayUser], BaseSession] = (
             self._session_cls.from_user
@@ -397,8 +400,8 @@ class BaseGateway(
         from ..group.bookmarks import LegacyBookmarks
         from ..group.room import LegacyMUC, LegacyParticipant
 
-        session_cls: Type[BaseSession] = cast(
-            Type[BaseSession], BaseSession.get_unique_subclass()
+        session_cls: type[BaseSession] = cast(
+            type[BaseSession], BaseSession.get_unique_subclass()
         )
         contact_cls = LegacyContact.get_self_or_unique_subclass()
         muc_cls = LegacyMUC.get_self_or_unique_subclass()
@@ -564,9 +567,13 @@ class BaseGateway(
 
     async def __on_privileges_advertised(self, _) -> None:
         await self.__upload_service_found.wait()
-        if self.xmpp["xep_0356"].granted_privileges[config.SERVER].iq.get(
-            self.plugin["xep_0363"].stanza.Request.namespace
-        ) in (IqPermission.SET, IqPermission.BOTH, IqPermission.GET):
+        if config.UPLOAD_SERVICE and self.xmpp["xep_0356"].granted_privileges[
+            config.SERVER
+        ].iq.get(self.plugin["xep_0363"].stanza.Request.namespace) in (
+            IqPermission.SET,
+            IqPermission.BOTH,
+            IqPermission.GET,
+        ):
             AttachmentMixin.PRIVILEGED_UPLOAD = True
 
     async def __setup_attachments(self) -> None:
@@ -580,9 +587,23 @@ class BaseGateway(
                 self.infer_real_domain()
             )
             if info_iq is None:
-                raise RuntimeError("Could not find upload service")
-            log.info("Auto-discovered upload service: %s", info_iq["from"])
-            config.UPLOAD_SERVICE = info_iq["from"]
+                if self.REGISTRATION_TYPE == RegistrationType.QRCODE:
+                    log.warning(
+                        "No method was configured for attachment and slidge "
+                        "could not automatically determine the JID of a usable upload service. "
+                        "Users likely won't be able to register since this network uses a "
+                        "QR-code based registration flow."
+                    )
+                if not config.USE_ATTACHMENT_ORIGINAL_URLS:
+                    log.warning(
+                        "Setting USE_ATTACHMENT_ORIGINAL_URLS to True since no method was configured "
+                        "for attachments and no upload service was found. NB: this does not work for all "
+                        "networks, especially for the E2EE attachments."
+                    )
+                    config.USE_ATTACHMENT_ORIGINAL_URLS = True
+            else:
+                log.info("Auto-discovered upload service: %s", info_iq["from"])
+                config.UPLOAD_SERVICE = info_iq["from"]
         self.__upload_service_found.set()
 
     def infer_real_domain(self) -> JID:
@@ -729,7 +750,7 @@ class BaseGateway(
                 "The admin controls that with the USER_JID_VALIDATOR option.",
             )
 
-    def send_raw(self, data: Union[str, bytes]):
+    def send_raw(self, data: str | bytes):
         # overridden from XMLStream to strip base64-encoded data from the logs
         # to make them more readable.
         if log.isEnabledFor(level=logging.DEBUG):
@@ -845,7 +866,7 @@ class BaseGateway(
         return reply
 
     async def user_prevalidate(
-        self, ifrom: JID, form_dict: dict[str, Optional[str]]
+        self, ifrom: JID, form_dict: dict[str, str | None]
     ) -> JSONSerializable | None:
         # Pre validate a registration form using the content of self.REGISTRATION_FIELDS
         # before passing it to the plugin custom validation logic.
@@ -855,8 +876,9 @@ class BaseGateway(
 
         return await self.validate(ifrom, form_dict)
 
+    @abc.abstractmethod
     async def validate(
-        self, user_jid: JID, registration_form: dict[str, Optional[str]]
+        self, user_jid: JID, registration_form: dict[str, str | None]
     ) -> JSONSerializable | None:
         """
         Validate a user's initial registration form.
@@ -928,8 +950,8 @@ class BaseGateway(
     async def confirm_qr(
         self,
         user_bare_jid: str,
-        exception: Optional[Exception] = None,
-        legacy_data: Optional[JSONSerializable] = None,
+        exception: Exception | None = None,
+        legacy_data: JSONSerializable | None = None,
     ) -> None:
         """
         This method is meant to be called to finalize QR code-based registration
