@@ -5,11 +5,11 @@ import os
 import shutil
 import stat
 import tempfile
+from collections.abc import Collection, Sequence
 from datetime import datetime
 from itertools import chain
 from mimetypes import guess_extension, guess_type
 from pathlib import Path
-from typing import Collection, Optional, Sequence, Union
 from urllib.parse import quote as urlquote
 from uuid import uuid4
 from xml.etree import ElementTree as ET
@@ -44,8 +44,8 @@ class AttachmentMixin(TextMessageMixin):
     async def __upload(
         self,
         file_path: Path,
-        file_name: Optional[str] = None,
-        content_type: Optional[str] = None,
+        file_name: str | None = None,
+        content_type: str | None = None,
     ) -> str:
         assert config.UPLOAD_SERVICE
 
@@ -97,8 +97,8 @@ class AttachmentMixin(TextMessageMixin):
     @staticmethod
     async def __no_upload(
         file_path: Path,
-        file_name: Optional[str] = None,
-        legacy_file_id: Optional[Union[str, int]] = None,
+        file_name: str | None = None,
+        legacy_file_id: str | int | None = None,
     ) -> tuple[Path, str]:
         file_id = str(uuid4()) if legacy_file_id is None else str(legacy_file_id)
         assert config.NO_UPLOAD_PATH is not None
@@ -260,7 +260,7 @@ class AttachmentMixin(TextMessageMixin):
         self,
         msg: Message,
         uploaded_url: str,
-        path: Optional[Path],
+        path: Path | None,
         attachment: LegacyAttachment,
         stored: Attachment,
     ) -> Thumbnail | None:
@@ -306,10 +306,10 @@ class AttachmentMixin(TextMessageMixin):
         self,
         msg: Message,
         uploaded_url: str,
-        path: Optional[Path],
+        path: Path | None,
         attachment: LegacyAttachment,
         stored: Attachment,
-        thumbnail: Optional[Thumbnail] = None,
+        thumbnail: Thumbnail | None = None,
     ) -> None:
         if stored.sfs is not None:
             msg.append(StatelessFileSharing(xml=ET.fromstring(stored.sfs)))
@@ -381,9 +381,9 @@ class AttachmentMixin(TextMessageMixin):
         msg: Message,
         legacy_msg_id: LegacyMessageType,
         uploaded_url: str,
-        caption: Optional[str] = None,
+        caption: str | None = None,
         carbon: bool = False,
-        when: Optional[datetime] = None,
+        when: datetime | None = None,
         correction: bool = False,
         **kwargs,
     ) -> list[Message]:
@@ -411,13 +411,13 @@ class AttachmentMixin(TextMessageMixin):
 
     def __get_base_message(
         self,
-        legacy_msg_id: Optional[LegacyMessageType] = None,
-        reply_to: Optional[MessageReference] = None,
-        when: Optional[datetime] = None,
-        thread: Optional[LegacyThreadType] = None,
+        legacy_msg_id: LegacyMessageType | None = None,
+        reply_to: MessageReference | None = None,
+        when: datetime | None = None,
+        thread: LegacyThreadType | None = None,
         carbon: bool = False,
         correction: bool = False,
-        mto: Optional[JID] = None,
+        mto: JID | None = None,
     ) -> Message:
         if correction:
             xmpp_ids = self._legacy_to_xmpp(legacy_msg_id)
@@ -449,13 +449,13 @@ class AttachmentMixin(TextMessageMixin):
     async def send_file(
         self,
         attachment: LegacyAttachment | Path | str,
-        legacy_msg_id: Optional[LegacyMessageType] = None,
+        legacy_msg_id: LegacyMessageType | None = None,
         *,
-        reply_to: Optional[MessageReference] = None,
-        when: Optional[datetime] = None,
-        thread: Optional[LegacyThreadType] = None,
+        reply_to: MessageReference | None = None,
+        when: datetime | None = None,
+        thread: LegacyThreadType | None = None,
         **kwargs,
-    ) -> tuple[Optional[str], list[Message]]:
+    ) -> tuple[str | None, list[Message]]:
         """
         Send a single file from this :term:`XMPP Entity`.
 
@@ -495,13 +495,13 @@ class AttachmentMixin(TextMessageMixin):
     async def __send_file(
         self,
         attachment: LegacyAttachment | Path | str,
-        legacy_msg_id: Optional[LegacyMessageType] = None,
+        legacy_msg_id: LegacyMessageType | None = None,
         *,
-        reply_to: Optional[MessageReference] = None,
-        when: Optional[datetime] = None,
-        thread: Optional[LegacyThreadType] = None,
+        reply_to: MessageReference | None = None,
+        when: datetime | None = None,
+        thread: LegacyThreadType | None = None,
         **kwargs,
-    ) -> tuple[Optional[str], list[Message]]:
+    ) -> tuple[str | None, list[Message]]:
         store_multi = kwargs.pop("store_multi", True)
         carbon = kwargs.pop("carbon", False)
         mto = kwargs.pop("mto", None)
@@ -569,13 +569,13 @@ class AttachmentMixin(TextMessageMixin):
 
     def __send_body(
         self,
-        body: Optional[str] = None,
-        legacy_msg_id: Optional[LegacyMessageType] = None,
-        reply_to: Optional[MessageReference] = None,
-        when: Optional[datetime] = None,
-        thread: Optional[LegacyThreadType] = None,
+        body: str | None = None,
+        legacy_msg_id: LegacyMessageType | None = None,
+        reply_to: MessageReference | None = None,
+        when: datetime | None = None,
+        thread: LegacyThreadType | None = None,
         **kwargs,
-    ) -> Optional[Message]:
+    ) -> Message | None:
         if body:
             return self.send_text(
                 body,
@@ -591,15 +591,15 @@ class AttachmentMixin(TextMessageMixin):
     async def send_files(
         self,
         attachments: Collection[LegacyAttachment],
-        legacy_msg_id: Optional[LegacyMessageType] = None,
-        body: Optional[str] = None,
+        legacy_msg_id: LegacyMessageType | None = None,
+        body: str | None = None,
         *,
-        reply_to: Optional[MessageReference] = None,
-        when: Optional[datetime] = None,
-        thread: Optional[LegacyThreadType] = None,
+        reply_to: MessageReference | None = None,
+        when: datetime | None = None,
+        thread: LegacyThreadType | None = None,
         body_first: bool = False,
         correction: bool = False,
-        correction_event_id: Optional[LegacyMessageType] = None,
+        correction_event_id: LegacyMessageType | None = None,
         **kwargs,
     ) -> None:
         # TODO: once the epic XEP-0385 vs XEP-0447 battle is over, pick
@@ -647,8 +647,8 @@ class AttachmentMixin(TextMessageMixin):
 
     def __store_multi(
         self,
-        legacy_msg_id: Optional[LegacyMessageType],
-        all_msgs: Sequence[Optional[Message]],
+        legacy_msg_id: LegacyMessageType | None,
+        all_msgs: Sequence[Message | None],
     ) -> None:
         if legacy_msg_id is None:
             return
