@@ -1,7 +1,7 @@
 import logging
 import uuid
 import warnings
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING
 
 from slixmpp import Iq, Message
 from slixmpp.plugins.xep_0004 import Form
@@ -29,7 +29,7 @@ PUBLISH_OPTIONS.add_field("pubsub#access_model", value="whitelist")
 class ChatStateMixin(MessageMaker):
     def __init__(self) -> None:
         super().__init__()
-        self.__last_chat_state: Optional[ChatState] = None
+        self.__last_chat_state: ChatState | None = None
 
     def _chat_state(self, state: ChatState, forced: bool = False, **kwargs) -> None:
         carbon = kwargs.get("carbon", False)
@@ -207,8 +207,8 @@ class InviteMixin(MessageMaker):
     def invite_to(
         self,
         muc: "LegacyMUC",
-        reason: Optional[str] = None,
-        password: Optional[str] = None,
+        reason: str | None = None,
+        password: str | None = None,
         **send_kwargs,
     ) -> None:
         """
