@@ -1,13 +1,13 @@
+import abc
 import asyncio
 import logging
+from collections.abc import Iterable
 from typing import (
     TYPE_CHECKING,
     Any,
     Generic,
-    Iterable,
     NamedTuple,
     Optional,
-    Type,
     Union,
     cast,
 )
@@ -23,7 +23,7 @@ from ..contact import LegacyContact, LegacyRoster
 from ..db.models import Contact, GatewayUser
 from ..group.bookmarks import LegacyBookmarks
 from ..group.room import LegacyMUC
-from ..util import ABCSubclassableOnceAtMost
+from ..util import SubclassableOnce
 from ..util.lock import NamedLockMixin
 from ..util.types import (
     LegacyGroupIdType,
@@ -44,15 +44,16 @@ if TYPE_CHECKING:
 
 
 class CachedPresence(NamedTuple):
-    status: Optional[str]
-    show: Optional[str]
+    status: str | None
+    show: str | None
     kwargs: dict[str, Any]
 
 
 class BaseSession(
     Generic[LegacyMessageType, RecipientType],
     NamedLockMixin,
-    metaclass=ABCSubclassableOnceAtMost,
+    SubclassableOnce,
+    abc.ABC,
 ):
     """
     The session of a registered :term:`User`.
@@ -85,7 +86,7 @@ class BaseSession(
     in which case the message ID is also a thread ID (and all messages are potential
     threads).
     """
-    SPECIAL_MSG_ID_PREFIX: Optional[str] = None
+    SPECIAL_MSG_ID_PREFIX: str | None = None
     """
     If you set this, XMPP message IDs starting with this won't be converted to legacy ID,
     but passed as is to :meth:`.on_react`, and usual checks for emoji restriction won't be
@@ -93,8 +94,8 @@ class BaseSession(
     This can be used to implement voting in polls in a hacky way.
     """
 
-    _roster_cls: Type[LegacyRoster]
-    _bookmarks_cls: Type[LegacyBookmarks]
+    _roster_cls: type[LegacyRoster]
+    _bookmarks_cls: type[LegacyBookmarks]
 
     def __init__(self, user: GatewayUser) -> None:
         super().__init__()
@@ -112,7 +113,7 @@ class BaseSession(
 
         self.thread_creation_lock = asyncio.Lock()
 
-        self.__cached_presence: Optional[CachedPresence] = None
+        self.__cached_presence: CachedPresence | None = None
 
         self.__tasks = set[asyncio.Task]()
 
@@ -143,7 +144,8 @@ class BaseSession(
         for task in self.__tasks:
             task.cancel()
 
-    async def login(self) -> Optional[str]:
+    @abc.abstractmethod
+    async def login(self) -> str | None:
         """
         Logs in the gateway user to the legacy network.
 
@@ -169,13 +171,13 @@ class BaseSession(
         chat: RecipientType,
         text: str,
         *,
-        reply_to_msg_id: Optional[LegacyMessageType] = None,
-        reply_to_fallback_text: Optional[str] = None,
+        reply_to_msg_id: LegacyMessageType | None = None,
+        reply_to_fallback_text: str | None = None,
         reply_to: Optional["Sender"] = None,
-        thread: Optional[LegacyThreadType] = None,
+        thread: LegacyThreadType | None = None,
         link_previews: Iterable[LinkPreview] = (),
-        mentions: Optional[list[Mention]] = None,
-    ) -> Optional[LegacyMessageType]:
+        mentions: list[Mention] | None = None,
+    ) -> LegacyMessageType | None:
         """
         Triggered when the user sends a text message from XMPP to a bridged entity, e.g.
         to ``translated_user_name@slidge.example.com``, or ``translated_group_name@slidge.example.com``
@@ -212,11 +214,11 @@ class BaseSession(
         url: str,
         *,
         http_response: aiohttp.ClientResponse,
-        reply_to_msg_id: Optional[LegacyMessageType] = None,
-        reply_to_fallback_text: Optional[str] = None,
-        reply_to: Optional[Union["LegacyContact", "LegacyParticipant"]] = None,
-        thread: Optional[LegacyThreadType] = None,
-    ) -> Optional[LegacyMessageType]:
+        reply_to_msg_id: LegacyMessageType | None = None,
+        reply_to_fallback_text: str | None = None,
+        reply_to: Union["LegacyContact", "LegacyParticipant"] | None = None,
+        thread: LegacyThreadType | None = None,
+    ) -> LegacyMessageType | None:
         """
         Triggered when the user sends a file using HTTP Upload (:xep:`0363`)
 
@@ -240,11 +242,11 @@ class BaseSession(
         chat: RecipientType,
         sticker: Sticker,
         *,
-        reply_to_msg_id: Optional[LegacyMessageType] = None,
-        reply_to_fallback_text: Optional[str] = None,
-        reply_to: Optional[Union["LegacyContact", "LegacyParticipant"]] = None,
-        thread: Optional[LegacyThreadType] = None,
-    ) -> Optional[LegacyMessageType]:
+        reply_to_msg_id: LegacyMessageType | None = None,
+        reply_to_fallback_text: str | None = None,
+        reply_to: Union["LegacyContact", "LegacyParticipant"] | None = None,
+        thread: LegacyThreadType | None = None,
+    ) -> LegacyMessageType | None:
         """
         Triggered when the user sends a file using HTTP Upload (:xep:`0363`)
 
@@ -261,7 +263,7 @@ class BaseSession(
         raise NotImplementedError
 
     async def on_active(
-        self, chat: RecipientType, thread: Optional[LegacyThreadType] = None
+        self, chat: RecipientType, thread: LegacyThreadType | None = None
     ):
         """
         Triggered when the user sends an 'active' chat state (:xep:`0085`)
@@ -274,7 +276,7 @@ class BaseSession(
     active = deprecated("BaseSession.active", on_active)
 
     async def on_inactive(
-        self, chat: RecipientType, thread: Optional[LegacyThreadType] = None
+        self, chat: RecipientType, thread: LegacyThreadType | None = None
     ):
         """
         Triggered when the user sends an 'inactive' chat state (:xep:`0085`)
@@ -287,7 +289,7 @@ class BaseSession(
     inactive = deprecated("BaseSession.inactive", on_inactive)
 
     async def on_composing(
-        self, chat: RecipientType, thread: Optional[LegacyThreadType] = None
+        self, chat: RecipientType, thread: LegacyThreadType | None = None
     ):
         """
         Triggered when the user starts typing in a legacy chat (:xep:`0085`)
@@ -300,7 +302,7 @@ class BaseSession(
     composing = deprecated("BaseSession.composing", on_composing)
 
     async def on_paused(
-        self, chat: RecipientType, thread: Optional[LegacyThreadType] = None
+        self, chat: RecipientType, thread: LegacyThreadType | None = None
     ):
         """
         Triggered when the user pauses typing in a legacy chat (:xep:`0085`)
@@ -313,7 +315,7 @@ class BaseSession(
     paused = deprecated("BaseSession.paused", on_paused)
 
     async def on_gone(
-        self, chat: RecipientType, thread: Optional[LegacyThreadType] = None
+        self, chat: RecipientType, thread: LegacyThreadType | None = None
     ):
         """
         Triggered when the user is "gone" in a legacy chat (:xep:`0085`)
@@ -327,7 +329,7 @@ class BaseSession(
         self,
         chat: RecipientType,
         legacy_msg_id: LegacyMessageType,
-        thread: Optional[LegacyThreadType] = None,
+        thread: LegacyThreadType | None = None,
     ):
         """
         Triggered when the user reads a message in a legacy chat. (:xep:`0333`)
@@ -352,10 +354,10 @@ class BaseSession(
         text: str,
         legacy_msg_id: LegacyMessageType,
         *,
-        thread: Optional[LegacyThreadType] = None,
+        thread: LegacyThreadType | None = None,
         link_previews: Iterable[LinkPreview] = (),
-        mentions: Optional[list[Mention]] = None,
-    ) -> Optional[LegacyMessageType]:
+        mentions: list[Mention] | None = None,
+    ) -> LegacyMessageType | None:
         """
         Triggered when the user corrects a message using :xep:`0308`
 
@@ -381,7 +383,7 @@ class BaseSession(
         chat: RecipientType,
         legacy_msg_id: LegacyMessageType,
         emojis: list[str],
-        thread: Optional[LegacyThreadType] = None,
+        thread: LegacyThreadType | None = None,
     ):
         """
         Triggered when the user sends message reactions (:xep:`0444`).
@@ -400,7 +402,7 @@ class BaseSession(
         self,
         chat: RecipientType,
         legacy_msg_id: LegacyMessageType,
-        thread: Optional[LegacyThreadType] = None,
+        thread: LegacyThreadType | None = None,
     ):
         """
         Triggered when the user retracts (:xep:`0424`) a message.
@@ -419,7 +421,7 @@ class BaseSession(
         show: PseudoPresenceShow,
         status: str,
         resources: dict[str, ResourceDict],
-        merged_resource: Optional[ResourceDict],
+        merged_resource: ResourceDict | None,
     ):
         """
         Called when the gateway component receives a presence, ie, when
@@ -440,7 +442,7 @@ class BaseSession(
 
     presence = deprecated("BaseSession.presence", on_presence)
 
-    async def on_search(self, form_values: dict[str, str]) -> Optional[SearchResult]:
+    async def on_search(self, form_values: dict[str, str]) -> SearchResult | None:
         """
         Triggered when the user uses Jabber Search (:xep:`0055`) on the component
 
@@ -456,11 +458,11 @@ class BaseSession(
 
     async def on_avatar(
         self,
-        bytes_: Optional[bytes],
-        hash_: Optional[str],
-        type_: Optional[str],
-        width: Optional[int],
-        height: Optional[int],
+        bytes_: bytes | None,
+        hash_: str | None,
+        type_: str | None,
+        width: int | None,
+        height: int | None,
     ) -> None:
         """
         Triggered when the user uses modifies their avatar via :xep:`0084`.
@@ -477,7 +479,7 @@ class BaseSession(
         raise NotImplementedError
 
     async def on_moderate(
-        self, muc: LegacyMUC, legacy_msg_id: LegacyMessageType, reason: Optional[str]
+        self, muc: LegacyMUC, legacy_msg_id: LegacyMessageType, reason: str | None
     ):
         """
         Triggered when the user attempts to retract a message that was sent in
@@ -511,7 +513,7 @@ class BaseSession(
         raise NotImplementedError
 
     async def on_invitation(
-        self, contact: LegacyContact, muc: LegacyMUC, reason: Optional[str]
+        self, contact: LegacyContact, muc: LegacyMUC, reason: str | None
     ) -> None:
         """
         Triggered when the user invites a :term:`Contact` to a legacy MUC via
@@ -738,8 +740,8 @@ class BaseSession(
 
     def send_gateway_status(
         self,
-        status: Optional[str] = None,
-        show=Optional[PresenceShows],
+        status: str | None = None,
+        show=PresenceShows | None,
         **kwargs,
     ) -> None:
         """
@@ -780,8 +782,8 @@ class BaseSession(
     def send_gateway_invite(
         self,
         muc: LegacyMUC,
-        reason: Optional[str] = None,
-        password: Optional[str] = None,
+        reason: str | None = None,
+        password: str | None = None,
     ) -> None:
         """
         Send an invitation to join a MUC, emanating from the gateway component.
@@ -840,7 +842,7 @@ class BaseSession(
             return await muc.get_participant(nick, create=False, fill_first=True)
         return muc
 
-    async def wait_for_ready(self, timeout: Optional[Union[int, float]] = 10):
+    async def wait_for_ready(self, timeout: int | float | None = 10):
         # """
         # Wait until session, contacts and bookmarks are ready
         #
@@ -853,7 +855,7 @@ class BaseSession(
             await asyncio.wait_for(asyncio.shield(self.ready), timeout)
             await asyncio.wait_for(asyncio.shield(self.contacts.ready), timeout)
             await asyncio.wait_for(asyncio.shield(self.bookmarks.ready), timeout)
-        except asyncio.TimeoutError:
+        except TimeoutError:
             raise XMPPError(
                 "recipient-unavailable",
                 "Legacy session is not fully initialized, retry later",
