@@ -4,9 +4,9 @@ import hashlib
 import logging
 import shutil
 import uuid
-from datetime import datetime, timedelta, timezone
+from collections.abc import Collection, Iterator
+from datetime import UTC, datetime, timedelta
 from mimetypes import guess_extension
-from typing import Collection, Iterator, Optional, Type
 
 import sqlalchemy as sa
 from slixmpp.exceptions import XMPPError
@@ -38,12 +38,12 @@ from .models import (
 
 
 class UpdatedMixin:
-    model: Type[Base] = NotImplemented
+    model: type[Base] = NotImplemented
 
     def __init__(self, session: Session) -> None:
         self.reset_updated(session)
 
-    def get_by_pk(self, session: Session, pk: int) -> Type[Base]:
+    def get_by_pk(self, session: Session, pk: int) -> type[Base]:
         stmt = select(self.model).where(self.model.id == pk)  # type:ignore
         return session.scalar(stmt)
 
@@ -90,11 +90,11 @@ class AvatarStore:
 
 
 LegacyToXmppType = (
-    Type[DirectMessages]
-    | Type[DirectThreads]
-    | Type[GroupMessages]
-    | Type[GroupThreads]
-    | Type[GroupMessagesOrigin]
+    type[DirectMessages]
+    | type[DirectThreads]
+    | type[GroupMessages]
+    | type[GroupThreads]
+    | type[GroupMessagesOrigin]
 )
 
 
@@ -180,7 +180,7 @@ class IdMapStore:
         return list(
             session.scalars(
                 select(type_.xmpp_id).filter_by(
-                    foreign_key=foreign_key, legacy_id=legacy_id
+                    foreign_key=foreign_key, legacy_id=str(legacy_id)
                 )
             )
         )
@@ -198,7 +198,7 @@ class IdMapStore:
     @staticmethod
     def _get_legacy(
         session: Session, foreign_key: int, xmpp_id: str, type_: LegacyToXmppType
-    ) -> Optional[str]:
+    ) -> str | None:
         return session.scalar(
             select(type_.legacy_id).filter_by(foreign_key=foreign_key, xmpp_id=xmpp_id)
         )
@@ -210,7 +210,7 @@ class IdMapStore:
         xmpp_id: str,
         group: bool,
         origin: bool = False,
-    ) -> Optional[str]:
+    ) -> str | None:
         if origin and group:
             return self._get_legacy(
                 session,
@@ -227,7 +227,7 @@ class IdMapStore:
 
     def get_thread(
         self, session: Session, foreign_key: int, xmpp_id: str, group: bool
-    ) -> Optional[str]:
+    ) -> str | None:
         return self._get_legacy(
             session,
             foreign_key,
@@ -311,7 +311,7 @@ class MAMStore:
         room_pk: int,
         message: HistoryMessage,
         archive_only: bool,
-        legacy_msg_id: Optional[str],
+        legacy_msg_id: str | None,
     ) -> None:
         source = (
             ArchivedMessageSource.BACKFILL
@@ -327,7 +327,7 @@ class MAMStore:
             existing = session.execute(
                 select(ArchivedMessage)
                 .where(ArchivedMessage.room_id == room_pk)
-                .where(ArchivedMessage.legacy_id == legacy_msg_id)
+                .where(ArchivedMessage.legacy_id == str(legacy_msg_id))
             ).scalar()
         if existing is not None:
             log.debug("Updating message %s in room %s", message.id, room_pk)
@@ -353,13 +353,13 @@ class MAMStore:
     def get_messages(
         session: Session,
         room_pk: int,
-        start_date: Optional[datetime] = None,
-        end_date: Optional[datetime] = None,
-        before_id: Optional[str] = None,
-        after_id: Optional[str] = None,
+        start_date: datetime | None = None,
+        end_date: datetime | None = None,
+        before_id: str | None = None,
+        after_id: str | None = None,
         ids: Collection[str] = (),
-        last_page_n: Optional[int] = None,
-        sender: Optional[str] = None,
+        last_page_n: int | None = None,
+        sender: str | None = None,
         flip: bool = False,
     ) -> Iterator[HistoryMessage]:
         q = select(ArchivedMessage).where(ArchivedMessage.room_id == room_pk)
@@ -415,13 +415,13 @@ class MAMStore:
                 msgs = msgs[-last_page_n:]
         for h in msgs:
             yield HistoryMessage(
-                stanza=str(h.stanza), when=h.timestamp.replace(tzinfo=timezone.utc)
+                stanza=str(h.stanza), when=h.timestamp.replace(tzinfo=UTC)
             )
 
     @staticmethod
     def get_first(
         session: Session, room_pk: int, with_legacy_id: bool = False
-    ) -> Optional[ArchivedMessage]:
+    ) -> ArchivedMessage | None:
         q = (
             select(ArchivedMessage)
             .where(ArchivedMessage.room_id == room_pk)
@@ -433,8 +433,8 @@ class MAMStore:
 
     @staticmethod
     def get_last(
-        session: Session, room_pk: int, source: Optional[ArchivedMessageSource] = None
-    ) -> Optional[ArchivedMessage]:
+        session: Session, room_pk: int, source: ArchivedMessageSource | None = None
+    ) -> ArchivedMessage | None:
         q = select(ArchivedMessage).where(ArchivedMessage.room_id == room_pk)
 
         if source is not None:
@@ -454,8 +454,8 @@ class MAMStore:
 
     @staticmethod
     def get_most_recent_with_legacy_id(
-        session: Session, room_pk: int, source: Optional[ArchivedMessageSource] = None
-    ) -> Optional[ArchivedMessage]:
+        session: Session, room_pk: int, source: ArchivedMessageSource | None = None
+    ) -> ArchivedMessage | None:
         q = (
             select(ArchivedMessage)
             .where(ArchivedMessage.room_id == room_pk)
@@ -471,7 +471,7 @@ class MAMStore:
         room_pk: int,
         after_id: str,
         source: ArchivedMessageSource = ArchivedMessageSource.LIVE,
-    ) -> Optional[ArchivedMessage]:
+    ) -> ArchivedMessage | None:
         after_timestamp = (
             session.query(ArchivedMessage.timestamp)
             .filter(ArchivedMessage.room_id == room_pk)
@@ -490,7 +490,7 @@ class MAMStore:
     @staticmethod
     def get_by_legacy_id(
         session: Session, room_pk: int, legacy_id: str
-    ) -> Optional[ArchivedMessage]:
+    ) -> ArchivedMessage | None:
         return (
             session.query(ArchivedMessage)
             .filter(ArchivedMessage.room_id == room_pk)
