@@ -1,9 +1,10 @@
 import logging
 import uuid
 import warnings
+from collections.abc import Collection
 from copy import copy
-from datetime import datetime, timezone
-from typing import TYPE_CHECKING, Collection, Optional
+from datetime import UTC, datetime
+from typing import TYPE_CHECKING, Optional
 
 from slixmpp import Iq, Message
 
@@ -57,6 +58,9 @@ class MessageArchive:
                     f"{uuid.uuid4()}@{participant.xmpp.boundjid.bare}"
                 )
 
+        if "replace" in msg:
+            legacy_msg_id = None
+
         with self.__store.session(expire_on_commit=False) as orm:
             if self.room.id is None:
                 self.room = self.__rooms_store.get(
@@ -78,7 +82,7 @@ class MessageArchive:
     def __to_bound(stored: ArchivedMessage):
         return HoleBound(
             stored.legacy_id,  # type:ignore
-            stored.timestamp.replace(tzinfo=timezone.utc),
+            stored.timestamp.replace(tzinfo=UTC),
         )
 
     def get_hole_bounds(self) -> tuple[HoleBound | None, HoleBound | None]:
@@ -111,17 +115,17 @@ class MessageArchive:
 
     def get_all(
         self,
-        start_date: Optional[datetime] = None,
-        end_date: Optional[datetime] = None,
-        before_id: Optional[str] = None,
-        after_id: Optional[str] = None,
+        start_date: datetime | None = None,
+        end_date: datetime | None = None,
+        before_id: str | None = None,
+        after_id: str | None = None,
         ids: Collection[str] = (),
-        last_page_n: Optional[int] = None,
-        sender: Optional[str] = None,
+        last_page_n: int | None = None,
+        sender: str | None = None,
         flip: bool = False,
     ):
         with self.__store.session() as orm:
-            for msg in self.__store.get_messages(
+            yield from self.__store.get_messages(
                 orm,
                 self.room.id,
                 before_id=before_id,
@@ -132,8 +136,7 @@ class MessageArchive:
                 start_date=start_date,
                 end_date=end_date,
                 flip=flip,
-            ):
-                yield msg
+            )
 
     async def send_metadata(self, iq: Iq) -> None:
         """
@@ -148,9 +151,7 @@ class MessageArchive:
         if messages:
             for x, m in [("start", messages[0]), ("end", messages[-1])]:
                 reply["mam_metadata"][x]["id"] = m.id
-                reply["mam_metadata"][x]["timestamp"] = m.sent_on.replace(
-                    tzinfo=timezone.utc
-                )
+                reply["mam_metadata"][x]["timestamp"] = m.sent_on.replace(tzinfo=UTC)
         else:
             reply.enable("mam_metadata")
         reply.send()
