@@ -1,6 +1,6 @@
-import abc
 import logging
-from typing import TYPE_CHECKING, Generic, Iterator, Literal, Optional, Type, overload
+from collections.abc import Iterator
+from typing import TYPE_CHECKING, Generic, Literal, overload
 
 from slixmpp import JID
 from slixmpp.exceptions import XMPPError
@@ -19,13 +19,13 @@ if TYPE_CHECKING:
 class LegacyBookmarks(
     Generic[LegacyGroupIdType, LegacyMUCType],
     NamedLockMixin,
-    metaclass=SubclassableOnce,
+    SubclassableOnce,
 ):
     """
     This is instantiated once per :class:`~slidge.BaseSession`
     """
 
-    _muc_cls: Type[LegacyMUCType]
+    _muc_cls: type[LegacyMUCType]
 
     def __init__(self, session: "BaseSession") -> None:
         self.session = session
@@ -110,7 +110,7 @@ class LegacyBookmarks(
                 )
             return await self.__update_if_needed(stored)
 
-    def by_jid_only_if_exists(self, jid: JID) -> Optional[LegacyMUCType]:
+    def by_jid_only_if_exists(self, jid: JID) -> LegacyMUCType | None:
         with self.xmpp.store.session() as orm:
             stored = (
                 orm.query(Room).filter_by(user=self.session.user, jid=jid).one_or_none()
@@ -179,7 +179,6 @@ class LegacyBookmarks(
 
         return muc
 
-    @abc.abstractmethod
     async def fill(self):
         """
         Establish a user's known groups.
@@ -219,5 +218,7 @@ class LegacyBookmarks(
             user_participant = await muc.get_user_participant()
             user_participant.kick(reason)
         with self.xmpp.store.session() as orm:
+            orm.add(muc.stored)
+            orm.refresh(muc.stored)
             orm.delete(muc.stored)
             orm.commit()
