@@ -4,7 +4,7 @@ import uuid
 import warnings
 from copy import copy
 from datetime import datetime
-from typing import TYPE_CHECKING, Any, Optional, Union
+from typing import TYPE_CHECKING, Any
 from xml.etree import ElementTree as ET
 
 import sqlalchemy as sa
@@ -46,7 +46,7 @@ class LegacyParticipant(
     MessageMixin,
     ChatterDiscoMixin,
     DBMixin,
-    metaclass=SubclassableOnce,
+    SubclassableOnce,
 ):
     """
     A legacy participant of a legacy group chat.
@@ -226,7 +226,7 @@ class LegacyParticipant(
             return
         self.send_last_presence(force=True, no_cache_online=True)
 
-    def __update_resource(self, unescaped_nickname: Optional[str]) -> None:
+    def __update_resource(self, unescaped_nickname: str | None) -> None:
         if not unescaped_nickname:
             self.stored.resource = ""
             if self.is_system:
@@ -298,9 +298,9 @@ class LegacyParticipant(
     def _make_presence(
         self,
         *,
-        last_seen: Optional[datetime] = None,
-        status_codes: Optional[set[int]] = None,
-        user_full_jid: Optional[JID] = None,
+        last_seen: datetime | None = None,
+        status_codes: set[int] | None = None,
+        user_full_jid: JID | None = None,
         **presence_kwargs,
     ):
         p = super()._make_presence(last_seen=last_seen, **presence_kwargs)
@@ -341,7 +341,7 @@ class LegacyParticipant(
         return self.nickname
 
     def __send_presence_if_needed(
-        self, stanza: Union[Message, Presence], full_jid: JID, archive_only: bool
+        self, stanza: Message | Presence, full_jid: JID, archive_only: bool
     ) -> None:
         if (
             archive_only
@@ -363,7 +363,7 @@ class LegacyParticipant(
     def _send(
         self,
         stanza: MessageOrPresenceTypeVar,
-        full_jid: Optional[JID] = None,
+        full_jid: JID | None = None,
         archive_only: bool = False,
         legacy_msg_id=None,
         initial_presence=False,
@@ -420,13 +420,13 @@ class LegacyParticipant(
                 )
         return item
 
-    def __add_nick_element(self, stanza: Union[Presence, Message]) -> None:
+    def __add_nick_element(self, stanza: Presence | Message) -> None:
         if (nick := self.nickname_no_illegal) != self.jid.resource:
             n = self.xmpp.plugin["xep_0172"].stanza.UserNick()
             n["nick"] = nick
             stanza.append(n)
 
-    def _get_last_presence(self) -> Optional[CachedPresence]:
+    def _get_last_presence(self) -> CachedPresence | None:
         own = super()._get_last_presence()
         if own is None and self.contact:
             return self.contact._get_last_presence()
@@ -436,7 +436,7 @@ class LegacyParticipant(
         self,
         full_jid: JID,
         nick_change: bool = False,
-        presence_id: Optional[str] = None,
+        presence_id: str | None = None,
     ) -> None:
         """
         Called when the user joins a MUC, as a mechanism
@@ -496,13 +496,13 @@ class LegacyParticipant(
         """
         self.muc.remove_participant(self, ban=True, reason=reason)
 
-    def get_disco_info(self, jid: OptJid = None, node: Optional[str] = None):
+    def get_disco_info(self, jid: OptJid = None, node: str | None = None):
         if self.contact is not None:
             return self.contact.get_disco_info()
         return super().get_disco_info()
 
     def moderate(
-        self, legacy_msg_id: LegacyMessageType, reason: Optional[str] = None
+        self, legacy_msg_id: LegacyMessageType, reason: str | None = None
     ) -> None:
         for i in self._legacy_to_xmpp(legacy_msg_id):
             m = self.muc.get_system_participant()._make_message()
@@ -519,8 +519,8 @@ class LegacyParticipant(
     def set_room_subject(
         self,
         subject: str,
-        full_jid: Optional[JID] = None,
-        when: Optional[datetime] = None,
+        full_jid: JID | None = None,
+        when: datetime | None = None,
         update_muc: bool = True,
     ) -> None:
         if update_muc:
@@ -543,7 +543,7 @@ class LegacyParticipant(
         self,
         thread: LegacyThreadType,
         subject: str | None,
-        when: Optional[datetime] = None,
+        when: datetime | None = None,
     ) -> None:
         msg = self._make_message()
         msg["thread"] = str(thread)
