@@ -5,18 +5,15 @@ import string
 import uuid
 import warnings
 from asyncio import Lock
+from collections.abc import AsyncIterator, Iterator
 from contextlib import asynccontextmanager
 from copy import copy
-from datetime import datetime, timedelta, timezone
+from datetime import UTC, datetime, timedelta
 from typing import (
     TYPE_CHECKING,
     Any,
-    AsyncIterator,
     Generic,
-    Iterator,
     Literal,
-    Optional,
-    Type,
     Union,
     overload,
 )
@@ -73,7 +70,7 @@ class LegacyMUC(
     ChatterDiscoMixin,
     ReactionRecipientMixin,
     ThreadRecipientMixin,
-    metaclass=SubclassableOnce,
+    SubclassableOnce,
 ):
     """
     A room, a.k.a. a Multi-User Chat.
@@ -129,7 +126,7 @@ class LegacyMUC(
 
     stored: Room
 
-    _participant_cls: Type[LegacyParticipantType]
+    _participant_cls: type[LegacyParticipantType]
 
     def __init__(self, session: "BaseSession", stored: Room) -> None:
         self.session = session
@@ -206,7 +203,7 @@ class LegacyMUC(
         return self.stored.n_participants
 
     @n_participants.setter
-    def n_participants(self, n_participants: Optional[int]) -> None:
+    def n_participants(self, n_participants: int | None) -> None:
         if self.stored.n_participants == n_participants:
             return
         self.update_stored_attribute(n_participants=n_participants)
@@ -222,13 +219,13 @@ class LegacyMUC(
         return f"<MUC #{self.stored.id} '{self.name}' ({self.stored.legacy_id} - {self.jid.user})'>"
 
     @property
-    def subject_date(self) -> Optional[datetime]:
+    def subject_date(self) -> datetime | None:
         if self.stored.subject_date is None:
             return None
-        return self.stored.subject_date.replace(tzinfo=timezone.utc)
+        return self.stored.subject_date.replace(tzinfo=UTC)
 
     @subject_date.setter
-    def subject_date(self, when: Optional[datetime]) -> None:
+    def subject_date(self, when: datetime | None) -> None:
         if self.subject_date == when:
             return
         self.update_stored_attribute(subject_date=when)
@@ -320,7 +317,7 @@ class LegacyMUC(
             orm.commit()
 
     async def get_participants(
-        self, affiliation: Optional[MucAffiliation] = None
+        self, affiliation: MucAffiliation | None = None
     ) -> AsyncIterator[LegacyParticipantType]:
         await self.__fill_participants()
         with self.xmpp.store.session(expire_on_commit=False, autoflush=False) as orm:
@@ -426,8 +423,8 @@ class LegacyMUC(
 
     async def backfill(
         self,
-        after: Optional[HoleBound] = None,
-        before: Optional[HoleBound] = None,
+        after: HoleBound | None = None,
+        before: HoleBound | None = None,
     ):
         """
         Override this if the legacy network provide server-side group archives.
@@ -436,12 +433,19 @@ class LegacyMUC(
         with the ``archive_only=True`` kwarg. This is only called once per slidge
         run for a given group.
 
-        :param after: Fetch messages after this one. If ``None``, it's up to you
-            to decide how far you want to go in the archive. If it's not ``None``,
-            it means slidge has some messages in this archive and you should really try
-            to complete it to avoid "holes" in the history of this group.
-        :param before: Fetch messages before this one. If ``None``, fetch all messages
-            up to the most recent one
+        :param after: Fetch messages after this one.
+            If ``None``, slidge's local archive was empty before start-up,
+            ie, no history was ever fetched for this room since the user registered.
+            It's up to gateway implementations to decide how far to fetch messages before
+            the user registered.
+            If not ``None``, slidge has some messages in this archive, and
+            the gateway shall try to fetch history up to (and excluding) this message
+            to avoid "holes" in the history of this group.
+        :param before: Fetch messages before this one.
+            If ``None``, the gateway shall fetch all messages up to the most recent one.
+            If not ``None``, slidge has already archived some live messages
+            it received during its lifetime, and there is no need to query the legacy
+            network for any message after (and including) this one.
         """
         raise NotImplementedError
 
@@ -475,7 +479,7 @@ class LegacyMUC(
         return self.type == MucType.CHANNEL
 
     @property
-    def subject_setter(self) -> Optional[str]:
+    def subject_setter(self) -> str | None:
         return self.stored.subject_setter
 
     @subject_setter.setter
@@ -605,16 +609,9 @@ class LegacyMUC(
         return user_muc_jid
 
     async def echo(
-        self, msg: Message, legacy_msg_id: Optional[LegacyMessageType] = None
+        self, msg: Message, legacy_msg_id: LegacyMessageType | None = None
     ) -> str:
-        origin_id = msg.get_origin_id()
-
         msg.set_from(self.user_muc_jid)
-        msg.set_id(msg.get_id())
-        if origin_id:
-            # because of slixmpp internal magic, we need to do this to ensure the origin_id
-            # is present
-            set_origin_id(msg, origin_id)
         if legacy_msg_id:
             msg["stanza_id"]["id"] = self.session.legacy_to_xmpp_msg_id(legacy_msg_id)
         else:
@@ -639,7 +636,7 @@ class LegacyMUC(
         self.__send_configuration_change((104,))
         self._send_room_presence()
 
-    def _send_room_presence(self, user_full_jid: Optional[JID] = None) -> None:
+    def _send_room_presence(self, user_full_jid: JID | None = None) -> None:
         if user_full_jid is None:
             tos = self.user_full_jids()
         else:
@@ -1125,10 +1122,10 @@ class LegacyMUC(
     async def __old_school_history(
         self,
         full_jid: JID,
-        maxchars: Optional[int] = None,
-        maxstanzas: Optional[int] = None,
-        seconds: Optional[int] = None,
-        since: Optional[datetime] = None,
+        maxchars: int | None = None,
+        maxstanzas: int | None = None,
+        seconds: int | None = None,
+        since: datetime | None = None,
     ) -> None:
         """
         Old-style history join (internal slidge use)
@@ -1142,11 +1139,11 @@ class LegacyMUC(
         """
         if since is None:
             if seconds is None:
-                start_date = datetime.now(tz=timezone.utc) - timedelta(days=1)
+                start_date = datetime.now(tz=UTC) - timedelta(days=1)
             else:
-                start_date = datetime.now(tz=timezone.utc) - timedelta(seconds=seconds)
+                start_date = datetime.now(tz=UTC) - timedelta(seconds=seconds)
         else:
-            start_date = since or datetime.now(tz=timezone.utc) - timedelta(days=1)
+            start_date = since or datetime.now(tz=UTC) - timedelta(days=1)
 
         for h_msg in self.archive.get_all(
             start_date=start_date, end_date=None, last_page_n=maxstanzas
@@ -1419,9 +1416,7 @@ class LegacyMUC(
                 reason="The gateway is configured to always send invitations for groups.",
             )
 
-    async def on_avatar(
-        self, data: Optional[bytes], mime: Optional[str]
-    ) -> Optional[Union[int, str]]:
+    async def on_avatar(self, data: bytes | None, mime: str | None) -> int | str | None:
         """
         Called when the user tries to set the avatar of the room from an XMPP
         client.
@@ -1450,8 +1445,8 @@ class LegacyMUC(
         self,
         contact: "LegacyContact",
         affiliation: MucAffiliation,
-        reason: Optional[str],
-        nickname: Optional[str],
+        reason: str | None,
+        nickname: str | None,
     ):
         """
         Triggered when the user requests changing the affiliation of a contact
@@ -1466,7 +1461,7 @@ class LegacyMUC(
         """
         raise NotImplementedError
 
-    async def on_kick(self, contact: "LegacyContact", reason: Optional[str]):
+    async def on_kick(self, contact: "LegacyContact", reason: str | None):
         """
         Triggered when the user requests changing the role of a contact
         to "none" for this group. Action commonly known as "kick".
@@ -1478,8 +1473,8 @@ class LegacyMUC(
 
     async def on_set_config(
         self,
-        name: Optional[str],
-        description: Optional[str],
+        name: str | None,
+        description: str | None,
     ):
         """
         Triggered when the user requests changing the room configuration.
@@ -1496,7 +1491,7 @@ class LegacyMUC(
         """
         raise NotImplementedError
 
-    async def on_destroy_request(self, reason: Optional[str]):
+    async def on_destroy_request(self, reason: str | None):
         """
         Triggered when the user requests room destruction.
 
@@ -1602,7 +1597,7 @@ def equals_zero(x):
         return x == 0
 
 
-def str_to_datetime_or_none(date: Optional[str]):
+def str_to_datetime_or_none(date: str | None):
     if date is None:
         return
     try:
