@@ -21,7 +21,6 @@ import logging.config
 import os
 import signal
 from pathlib import Path
-from typing import Type
 
 import configargparse
 
@@ -62,9 +61,10 @@ class SigTermInterrupt(Exception):
 
 def get_configurator(from_entrypoint: bool = False) -> MainConfig:
     p = configargparse.ArgumentParser(
-        default_config_files=os.getenv(
-            "SLIDGE_CONF_DIR", "/etc/slidge/conf.d/*.conf"
-        ).split(":"),
+        default_config_files=[
+            f"{p}/*"
+            for p in os.getenv("SLIDGE_CONF_DIR", "/etc/slidge/conf.d/").split(":")
+        ],
         description=__doc__,
     )
     p.add_argument(
@@ -172,7 +172,7 @@ def main(module_name: str | None = None) -> None:
 
     migrate()
 
-    gw_cls: Type[BaseGateway] = BaseGateway.get_unique_subclass()  # type:ignore[assignment]
+    gw_cls: type[BaseGateway] = BaseGateway.get_unique_subclass()  # type:ignore[assignment]
     store = SlidgeStore(
         get_engine(
             config.DB_URL,
