@@ -1,5 +1,4 @@
 from .util import (
-    ABCSubclassableOnceAtMost,
     SubclassableOnce,
     is_valid_phone_number,
     replace_mentions,
@@ -8,7 +7,6 @@ from .util import (
 
 __all__ = [
     "SubclassableOnce",
-    "ABCSubclassableOnceAtMost",
     "is_valid_phone_number",
     "replace_mentions",
     "strip_illegal_chars",
