@@ -1,7 +1,6 @@
 import uuid
 from copy import copy
-from datetime import datetime, timezone
-from typing import Optional, Union
+from datetime import UTC, datetime
 from xml.etree import ElementTree as ET
 
 from slixmpp import Message
@@ -33,9 +32,7 @@ def set_client_namespace(stanza: StanzaBase) -> None:
 
 
 class HistoryMessage:
-    def __init__(
-        self, stanza: Union[Message, str], when: Optional[datetime] = None
-    ) -> None:
+    def __init__(self, stanza: Message | str, when: datetime | None = None) -> None:
         if isinstance(stanza, str):
             from_db = True
             stanza = Message(xml=ET.fromstring(stanza))
@@ -43,9 +40,7 @@ class HistoryMessage:
             from_db = False
 
         self.id = stanza["stanza_id"]["id"] or uuid.uuid4().hex
-        self.when: datetime = (
-            when or stanza["delay"]["stamp"] or datetime.now(tz=timezone.utc)
-        )
+        self.when: datetime = when or stanza["delay"]["stamp"] or datetime.now(tz=UTC)
 
         if not from_db:
             del stanza["delay"]
