@@ -1,7 +1,7 @@
 import logging
 from functools import cached_property
-from types import GenericAlias
-from typing import Any, Optional, Union, get_args, get_origin, get_type_hints
+from types import GenericAlias, UnionType
+from typing import Any, Union, get_args, get_origin, get_type_hints
 
 import configargparse
 
@@ -98,7 +98,7 @@ class ConfigModule:
     def __init__(
         self,
         config_obj: Any,
-        parser: Optional[configargparse.ArgumentParser] = None,
+        parser: configargparse.ArgumentParser | None = None,
         skip_options: tuple[str, ...] = (),
     ) -> None:
         self.config_obj = config_obj
@@ -120,7 +120,7 @@ class ConfigModule:
         }
 
     def set_conf(
-        self, argv: Optional[list[str]] = None
+        self, argv: list[str] | None = None
     ) -> tuple[configargparse.Namespace, list[str]]:
         if argv is not None:
             # this is ugly, but necessary because for plugin config, we used
@@ -139,7 +139,7 @@ class ConfigModule:
                 force_keep = False
                 if "=" in a:
                     real_name, _value = a.split("=")
-                    opt: Optional[Option] = options_long.get(
+                    opt: Option | None = options_long.get(
                         _argv_to_option_name(real_name)
                     )
                     if opt and opt.type is bool:
@@ -205,9 +205,10 @@ class ConfigModule:
 
 
 def _is_optional(t: Any) -> bool:
-    if get_origin(t) is Union:
+    origin = get_origin(t)
+    if origin is Union or origin is UnionType:
         args = get_args(t)
-        if len(args) == 2 and isinstance(None, args[1]):
+        if len(args) == 2 and type(None) in args:
             return True
     return False
 
