@@ -1,7 +1,8 @@
 import asyncio
 import logging
+from collections.abc import AsyncIterator, Hashable
 from contextlib import asynccontextmanager
-from typing import Any, AsyncIterator, Hashable
+from typing import Any
 
 
 class NamedLockMixin:
