@@ -1,8 +1,8 @@
 # type:ignore
+import os
 import tempfile
 import types
 from pathlib import Path
-from typing import Optional, Union
 from xml.dom.minidom import parseString
 
 try:
@@ -47,6 +47,7 @@ from ..db import SlidgeStore
 from ..db.avatar import avatar_cache
 from ..db.meta import Base
 from ..db.models import Contact, GatewayUser
+from ..util import SubclassableOnce
 
 
 class SlixTestPlus(SlixTest):
@@ -108,9 +109,9 @@ class SlixTestPlus(SlixTest):
             test = Matcher(criteria)
             self.assertTrue(
                 test.match(stanza),
-                "Stanza did not match using %s method:\n" % method
-                + "Criteria:\n%s\n" % str(criteria)
-                + "Stanza:\n%s" % str(stanza),
+                f"Stanza did not match using {method} method:\n"
+                + f"Criteria:\n{str(criteria)}\n"
+                + f"Stanza:\n{str(stanza)}",
             )
         else:
             stanza_class = stanza.__class__
@@ -155,18 +156,18 @@ class SlixTestPlus(SlixTest):
                 stanza3.values = values
 
                 debug = "Three methods for creating stanzas do not match.\n"
-                debug += "Given XML:\n%s\n" % highlight(tostring(xml))
-                debug += "Given stanza:\n%s\n" % format_stanza(stanza)
-                debug += "Generated stanza:\n%s\n" % highlight(tostring(stanza2.xml))
-                debug += "Second generated stanza:\n%s\n" % highlight(
-                    tostring(stanza3.xml)
+                debug += f"Given XML:\n{highlight(tostring(xml))}\n"
+                debug += f"Given stanza:\n{format_stanza(stanza)}\n"
+                debug += f"Generated stanza:\n{highlight(tostring(stanza2.xml))}\n"
+                debug += (
+                    f"Second generated stanza:\n{highlight(tostring(stanza3.xml))}\n"
                 )
                 result = self.compare(xml, stanza.xml, stanza2.xml, stanza3.xml)
             else:
                 debug = "Two methods for creating stanzas do not match.\n"
-                debug += "Given XML:\n%s\n" % highlight(tostring(xml))
-                debug += "Given stanza:\n%s\n" % format_stanza(stanza)
-                debug += "Generated stanza:\n%s\n" % highlight(tostring(stanza2.xml))
+                debug += f"Given XML:\n{highlight(tostring(xml))}\n"
+                debug += f"Given stanza:\n{format_stanza(stanza)}\n"
+                debug += f"Generated stanza:\n{highlight(tostring(stanza2.xml))}\n"
                 result = self.compare(xml, stanza.xml, stanza2.xml)
             stanza_class.namespace = old_ns
 
@@ -180,9 +181,7 @@ class SlixTestPlus(SlixTest):
                     )
             self.assertTrue(result, debug)
 
-    def next_sent(
-        self, timeout: float = 0.05
-    ) -> Optional[Union[Message, Iq, Presence]]:
+    def next_sent(self, timeout: float = 0.05) -> Message | Iq | Presence | None:
         self.wait_for_send_queue()
         sent = self.xmpp.socket.next_sent(timeout=timeout)
         if sent is None:
@@ -194,7 +193,7 @@ class SlixTestPlus(SlixTest):
 
 
 class SlidgeTest(SlixTestPlus):
-    plugin: Union[types.ModuleType, dict]
+    plugin: types.ModuleType | dict
 
     class Config:
         jid = "aim.shakespeare.lit"
@@ -212,36 +211,43 @@ class SlidgeTest(SlixTestPlus):
     def setUpClass(cls) -> None:
         for k, v in vars(cls.Config).items():
             setattr(config, k.upper(), v)
+        if hasattr(cls, "plugin"):
+            subclasses = SubclassableOnce._SubclassableOnce__subclasses
+            subclasses.update(
+                {
+                    BaseGateway: find_subclass(cls.plugin, BaseGateway),
+                    BaseSession: find_subclass(cls.plugin, BaseSession),
+                    LegacyRoster: find_subclass(cls.plugin, LegacyRoster, base_ok=True),
+                    LegacyContact: find_subclass(
+                        cls.plugin, LegacyContact, base_ok=True
+                    ),
+                    LegacyMUC: find_subclass(cls.plugin, LegacyMUC, base_ok=True),
+                    LegacyBookmarks: find_subclass(
+                        cls.plugin, LegacyBookmarks, base_ok=True
+                    ),
+                }
+            )
+            for subclass in subclasses.values():
+                if subclass is not None:
+                    subclass.__abstractmethods__ = set()
 
     def setUp(self):
-        if hasattr(self, "plugin"):
-            BaseGateway._subclass = find_subclass(self.plugin, BaseGateway)
-            BaseSession._subclass = find_subclass(self.plugin, BaseSession)
-            LegacyRoster._subclass = find_subclass(
-                self.plugin, LegacyRoster, base_ok=True
-            )
-            LegacyContact._subclass = find_subclass(
-                self.plugin, LegacyContact, base_ok=True
-            )
-            LegacyMUC._subclass = find_subclass(self.plugin, LegacyMUC, base_ok=True)
-            LegacyBookmarks._subclass = find_subclass(
-                self.plugin, LegacyBookmarks, base_ok=True
-            )
-
         # workaround for duplicate output of sql alchemy's log, cf
         # https://stackoverflow.com/a/76498428/5902284
         from sqlalchemy import log as sqlalchemy_log
+        from sqlalchemy.pool import StaticPool
 
         sqlalchemy_log._add_default_handler = lambda x: None
-
-        engine = self.db_engine = create_engine("sqlite+pysqlite:///:memory:")
+        db_url = os.getenv("SLIDGETEST_DB_URL", "sqlite+pysqlite:///:memory:")
+        engine = self.db_engine = create_engine(
+            db_url, poolclass=StaticPool if db_url.startswith("postgresql+") else None
+        )
         Base.metadata.create_all(engine)
         BaseGateway.store = SlidgeStore(engine)
         BaseGateway._test_mode = True
-        try:
-            self.xmpp = BaseGateway.get_self_or_unique_subclass()()
-        except Exception:
-            raise
+
+        self.xmpp = BaseGateway.get_self_or_unique_subclass()()
+
         self.xmpp.TEST_MODE = True
         avatar_cache.store = self.xmpp.store.avatars
         avatar_cache.set_dir(Path(tempfile.mkdtemp()))
@@ -277,10 +283,57 @@ class SlidgeTest(SlixTestPlus):
         self.xmpp.use_presence_ids = False
         Error.namespace = "jabber:component:accept"
 
+    def _add_slidge_user(
+        self,
+        jid: str | JID = "romeo@shakespeare.lit",
+        legacy_module_data=None,
+        preferences=None,
+    ) -> None:
+        with self.xmpp.store.session() as orm:
+            user = GatewayUser(
+                jid=JID(jid),
+                legacy_module_data=legacy_module_data or {},
+                preferences=preferences
+                or {"sync_avatar": False, "sync_presence": False},
+            )
+            orm.add(user)
+            orm.commit()
+        self.run_coro(
+            self.xmpp._BaseGateway__dispatcher._on_user_register(Iq(sfrom=JID(jid)))
+        )
+        welcome = self.next_sent()
+        assert welcome["body"]
+        stanza = self.next_sent()
+        assert "logging in" in stanza["status"].lower(), stanza
+        stanza = self.next_sent()
+        assert "syncing contacts" in stanza["status"].lower(), stanza
+        stanza = self.next_sent()
+        assert "syncing groups" in stanza["status"].lower(), stanza
+        probe = self.next_sent()
+        assert probe.get_type() == "probe"
+        stanza = self.next_sent()
+        assert stanza["status"].lower()
+
+    def user_session(self, jid: str | JID = "romeo@shakespeare.lit") -> BaseSession:
+        return BaseSession.get_self_or_unique_subclass().from_jid(JID(jid))
+
+    def get_joined_muc(
+        self,
+        legacy_id: str | int,
+        user_jid: str | JID = "romeo@shakespeare.lit",
+        resource: str = "gajim",
+    ) -> LegacyMUC:
+        muc: LegacyMUC = self.run_coro(
+            self.user_session(user_jid).bookmarks.by_legacy_id(legacy_id)
+        )
+        muc.add_user_resource(resource)
+        return muc
+
     def tearDown(self) -> None:
         self.db_engine.echo = False
         super().tearDown()
         Base.metadata.drop_all(self.xmpp.store._engine)
+        self.db_engine.dispose()
         _sessions.clear()
 
     def setup_logged_session(self, n_contacts: int = 0) -> None:
