@@ -2,7 +2,8 @@
 User actions
 """
 
-from typing import TYPE_CHECKING, Iterable, Optional, Union
+from collections.abc import Iterable
+from typing import TYPE_CHECKING, Union
 
 from slidge import BaseSession, GatewayUser
 from slidge.util.types import LinkPreview, Mention
@@ -32,13 +33,13 @@ class Session(BaseSession[str, Recipient]):
         chat: Recipient,
         text: str,
         *,
-        reply_to_msg_id: Optional[str] = None,
-        reply_to_fallback_text: Optional[str] = None,
-        reply_to: Optional[Sender] = None,  # type:ignore
-        thread: Optional[str] = None,  # type:ignore
+        reply_to_msg_id: str | None = None,
+        reply_to_fallback_text: str | None = None,
+        reply_to: Sender | None = None,  # type:ignore
+        thread: str | None = None,  # type:ignore
         link_previews: Iterable[LinkPreview] = (),
-        mentions: Optional[list[Mention]] = None,
-    ) -> Optional[str]:
+        mentions: list[Mention] | None = None,
+    ) -> str | None:
         if chat.is_group:
             assert isinstance(chat, MUC)
             msg = await self.legacy_client.send_group_msg(text, chat.legacy_id)
