@@ -192,6 +192,36 @@ class TestContactAvatar(BaseNoMUC, AvatarFixtureMixin):
         self.run_coro(juliet.set_avatar(Avatar(url=self.avatar_url, unique_id="someid")))
         assert self.next_sent() is None
 
+    def test_url_no_headers_remote_has_not_changed(self):
+        juliet = self.juliet
+        juliet.avatar = Avatar(url=self.avatar_url_no_headers)
+        self.run_coro(juliet._set_avatar_task)
+        self.__assert_publish(rewritten=True)
+
+        juliet = self.juliet
+        juliet.avatar = Avatar(url=self.avatar_url_no_headers)
+        self.run_coro(juliet._set_avatar_task)
+        # TODO: no reason to broadcast the avatar here, but we rely on HTTP headers
+        #       to not re-publish. This is probably in the realm of micro-optimisation
+        #       and does not matter much though…
+        self.__assert_publish(rewritten=True)
+
+    def test_url_no_headers_remote_has_changed(self):
+        juliet = self.juliet
+        juliet.avatar = Avatar(url=self.avatar_url_no_headers)
+        self.run_coro(juliet._set_avatar_task)
+        self.__assert_publish(rewritten=True)
+
+        with self.switch_avatar():
+            juliet = self.juliet
+            juliet.avatar = Avatar(url=self.avatar_url_no_headers)
+            self.run_coro(juliet._set_avatar_task)
+            assert self.next_sent() is not None
+
+            juliet = self.juliet
+            juliet.avatar = Avatar(url=self.avatar_url_no_headers)
+            self.run_coro(juliet._set_avatar_task)
+            assert self.next_sent() is not None
 
 class MUC(LegacyMUC):
     type = MucType.GROUP
