@@ -284,3 +284,18 @@ def test_unrecognized_after_boolean():
     ns, rest = configurator.set_conf(["--prout", "--joujoujou"])
     assert Config.PROUT
     assert rest == ["--joujoujou"]
+
+
+def test_custom_conf_dirs(monkeypatch, tmp_path_factory):
+    dir1 = tmp_path_factory.mktemp("dir1")
+    dir2 = tmp_path_factory.mktemp("dir2")
+
+    (dir1 / "file1.ini").write_text("jid=some-jid\nlegacy-module=prout")
+    (dir2 / "file2.ini").write_text("secret=some-secret")
+
+    monkeypatch.setenv("SLIDGE_CONF_DIR", f"{dir1}:{dir2}")
+    conf = main.get_configurator()
+    args = conf.parser.parse_args([])
+    assert args.jid == "some-jid"
+    assert args.secret == "some-secret"
+    assert args.legacy_module == "prout"
