@@ -774,3 +774,38 @@ class TestSession(AvatarFixtureMixin, SlidgeTest):
             legacy_msg_id = on_retract.call_args[0][1]
             assert contact.jid.node == "juliet"
             assert legacy_msg_id == "legacy-id"
+
+    def test_reject_omemo(self):
+        elements = [
+            "<encrypted />",
+            "<encrypted xmlns='whatever' />",
+            "<encrypted>blabla</encrypted>",
+            "<encrypted xmlns='whatever'>blabla</encrypted>",
+        ]
+        for el in elements:
+            with unittest.mock.patch("slidge.core.session.BaseSession.on_text") as on_text:
+                self.recv(  # language=XML
+                    f"""
+            <message type="chat"
+                     to="juliet@aim.shakespeare.lit"
+                     from="romeo@montague.lit/whatevs"
+                     id="origin-id">{el}
+            <body>your client blablabla</body></message>
+            """
+                )
+                on_text.assert_not_awaited()
+                self.send(  # language=XML
+                    """
+            <message xmlns="jabber:component:accept"
+                     type="error"
+                     to="romeo@montague.lit/whatevs"
+                     from="juliet@aim.shakespeare.lit"
+                     id="origin-id">
+              <error type="modify">
+                <bad-request xmlns="urn:ietf:params:xml:ns:xmpp-stanzas" />
+                <text xmlns="urn:ietf:params:xml:ns:xmpp-stanzas">You cannot send encrypted messages through this gateway</text>
+              </error>
+            </message>
+            """,
+                    use_values=False,
+                )
